using System.Collections.Concurrent;
using System.Reflection;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;

namespace Infrastructure.Utils.Xml;

public static class XmlSchemaSetHelper
{
    private static readonly ConcurrentDictionary<string, XmlSchemaSet> Cache = new();

    public static XmlSchemaSet SchemaSetByName(XName name, Assembly assembly, XmlValidationOptions validationOptions) =>
        Cache.GetOrAdd(name.ToString(), _ => SchemaSet(name, assembly, validationOptions));

    private static XmlSchemaSet SchemaSet(XName name, Assembly assembly, XmlValidationOptions validationOptions)
    {
        var resources = LoadXmlSchemaResources(name.LocalName, assembly);
        var resource = resources.FirstOrDefault(resource =>
        {
            var schemaVersionMatching = validationOptions?.SchemaVersion == default
                                        || resource.Schema.Version == validationOptions.SchemaVersion;
            var namesMatching = resource.Schema.TargetNamespace == name.NamespaceName &&
                                resource.Schema.Id == name.LocalName;
            return namesMatching && schemaVersionMatching;
        });

        return resource is not null ? CreateSchemaSet(resource, assembly) : null;
    }

    private static IEnumerable<XmlSchemaResource> LoadXmlSchemaResources(string name, Assembly assembly)
    {
        return assembly.GetManifestResourceNames()
            .Where(resourceName => resourceName.Contains($"{name}.xsd"))
            .Select(resourceName =>
            {
                using var resourceStream = assembly.GetManifestResourceStream(resourceName)
                                           ?? throw new InvalidOperationException();

                using var textReader = new XmlTextReader(resourceStream);
                var schema = XmlSchema.Read(textReader, (_, _) => { });
                return new XmlSchemaResource(resourceName, schema);
            });
    }

    private static XmlSchemaSet CreateSchemaSet(XmlSchemaResource schemaResource, Assembly assembly)
    {
        var resourceName = schemaResource.ResourceName;
        var xsdName = schemaResource.Schema.Id;

        var xmlResolver = new ResourceXmlResolver(assembly)
        {
            Prefix = resourceName[
                ..resourceName.LastIndexOf(xsdName ?? string.Empty, StringComparison.InvariantCultureIgnoreCase)]
        };

        var schemaSet = new XmlSchemaSet
        {
            XmlResolver = xmlResolver
        };

        schemaSet.Add(schemaResource.Schema);
        return schemaSet;
    }
}