﻿using System.Reflection;
using System.Xml.Linq;
using System.Xml.Schema;

namespace Infrastructure.Utils.Xml;

public class XmlValidator : IXmlValidator
{
    public XmlValidationResult ValidateDocument(string xmlDocument, Assembly assembly,
        XmlValidationOptions options = default)
    {
        using var reader = new StringReader(xmlDocument);
        return ValidateDocument(XDocument.Load(reader), assembly, options);
    }

    public XmlValidationResult ValidateDocument<T>(string xmlDocument, XmlValidationOptions options = default) =>
        ValidateDocument(typeof(T), xmlDocument, options);

    public XmlValidationResult ValidateDocument(Type type, string xmlDocument,
        XmlValidationOptions options = default) =>
        ValidateDocument(xmlDocument, type.Assembly, options);

    private XmlValidationResult ValidateDocument(XDocument xmlDocument, Assembly assembly,
        XmlValidationOptions options = default)
    {
        var errors = new List<ValidationEventArgs>();
        var result = new XmlValidationResult(xmlDocument, errors);

        if (xmlDocument == null)
        {
            result.Errors.Add("The document is empty");
            return result;
        }

        var rootName = xmlDocument.Root?.Name;
        var xsdName = rootName?.LocalName;
        var documentNamespace = rootName?.NamespaceName;

        if (options?.RequiredNamespace != default && options.RequiredNamespace != documentNamespace)
        {
            result.Errors.Add(
                $"The documents namespace '{documentNamespace}' does not match namespace '{options.RequiredNamespace}'");
            return result;
        }

        var schemaSet = XmlSchemaSetHelper.SchemaSetByName(rootName, assembly, options);

        if (schemaSet is null)
        {
            result.Errors.Add($"No schemas found with name {xsdName}");
            return result;
        }

        xmlDocument.Validate(schemaSet, (_, e) => errors.Add(e));

        return new XmlValidationResult(xmlDocument, errors);
    }
}